#!/bin/sh

TST_CNT=9
TST_SETUP=setup
TST_TESTFUNC=test
TST_NEEDS_TMPDIR=1
TST_NEEDS_ROOT=1
TST_NEEDS_CMDS="awk bc numactl numastat"
PAGESIZE=

#使用awk命令匹配numastat命令的节点值字段
# $1 - 进程号
# $2 - 节点号
get_node_index()
{
       local pid=$1
       local nid="Node $2"
       echo $(numastat -p $pid | sed '3q;d' | awk -F '[[:space:]][[:space:]]+' \
               -v node="$nid" '{ for (i = 1; i <= NF; ++i) if($i==node) print i; exit }')
}

#转换‘numastat -p’命令中给定节点的值，将其乘以大小
# $1 - 进程号
# $2 - 节点号
# $3 - 乘积，例如1024MB
get_mem_cur()
{
	local pid=$1
	local index=$(echo "$(get_node_index $pid $2)")
	local size=$3
	local numstat=$(numastat -p $pid |awk '/^Total/ {print $'$index'}')

	if [ -z "$numstat" ]; then
		echo 0
		return
	fi

	echo $(echo "$numstat * $size" | bc)
}

check_for_support_numa()
{
	local pid=$1

	local state=$(awk '{print $3}' /proc/$pid/stat)

	if [ $state = 'T' ]; then
		return 0
	fi

	return 1
}

setup()
{
	export MB=$((1024*1024))
	export PAGE_SIZE=$(awk '/Pagesize:/ {print $2}' /proc/meminfo)
	export HPAGE_SIZE=$(awk '/Hugepagesize:/ {print $2}' /proc/meminfo)

	total_nodes=0

	nodes_list=$(numactl --show | grep nodebind | cut -d ':' -f 2)
	for node in $nodes_list; do
		total_nodes=$((total_nodes+1))
	done

	echo "The system contains $total_nodes nodes: $nodes_list"

	if [ $total_nodes -le 1 ]; then
		echo "SUT does not support NUMA policy or not a NUMA machine"
	fi
}

# 在特定节点上申请内存
test1()
{
	local mem_curr

	for node in $nodes_list; do
		numactl --cpunodebind=$node --membind=$node ./support_numa alloc_1MB &
		pid=$!

		#TST_RETRY_FUNC "check_for_support_numa $pid" 0
		check_for_support_numa $pid

		mem_curr=$(get_mem_cur $pid $node $MB)
		if [ $(echo "$mem_curr < $MB" | bc) -eq 1 ]; then
			echo "Fail:NUMA memory allocated in node$node is less than expected"
			kill -CONT $pid >/dev/null 2>&1
			return
		fi

		kill -CONT $pid >/dev/null 2>&1
	done

	echo "Pass:NUMA local node and memory affinity"
}

# 在偏好节点上申请内存
test2()
{
	local mem_curr
	local cnt=1

	for node in $nodes_list; do

		if [ $cnt -eq $total_nodes ]; then   #wrap up for last node
			Preferred_node=$(echo $nodes_list | cut -d ' ' -f 1)
		else
			# always next node is preferred node
			Preferred_node=$(echo $nodes_list | cut -d ' ' -f $((cnt+1)))
		fi

		numactl --cpunodebind=$node --preferred=$Preferred_node ./support_numa alloc_1MB &
		pid=$!

		#TST_RETRY_FUNC "check_for_support_numa $pid" 0
		check_for_support_numa $pid

		mem_curr=$(get_mem_cur $pid $Preferred_node $MB)
		if [ $(echo "$mem_curr < $MB" |bc ) -eq 1 ]; then
		        echo "Fail:NUMA memory allocated in node$Preferred_node is less than expected"
			kill -CONT $pid >/dev/null 2>&1
			return
		fi

		cnt=$((cnt+1))
		kill -CONT $pid >/dev/null 2>&1
	done

	echo "Pass:NUMA preferred node policy"
}


check_ltp_numa_test8_log()
{
	grep -m1 -q '.' ltp_numa_test8.log
}


#获取可用内存节点及其可用内存,返回示例如下
#               available: 2 nodes (0-1)
#               node 0 size: 7808 MB
#               node 0 free: 7457 MB
#               node 1 size: 5807 MB
#               node 1 free: 5731 MB
#               node distances:
#               node   0   1
#                 0:  10  20
#                 1:  20  10
#
test3()
{
	RC=0

	numactl --hardware > gavail_nodes
	RC=$(awk '{ if ( NR == 1 ) {print $1;} }' gavail_nodes)
	if [ $RC = "available:" ]; then
		RC=$(awk '{ if ( NR == 1 ) {print $3;} }' gavail_nodes)
		if [ $RC = "nodes" ]; then
			RC=$(awk '{ if ( NR == 1 ) {print $2;} }' gavail_nodes)
			echo "Pass:NUMA policy on lib NUMA_NODE_SIZE API"
		else
			echo "Fail:Failed with NUMA policy"
		fi
	else
		echo "Fail:Failed with NUMA policy"
	fi
}

numa_run()
{
	make
	setup
	test1
	test2
	test3
}

numa_run



